#! /bin/bash
#
#   Copyright (c) 2010 SCM Microsystems GmbH , Germany
#  $Id: $
#   Authors: Benjamin Winter 
# 
# Version: 1.0      14.01.2010
#
# Return values:
# 0 - success
# 1 - generic or unspecified error
# 2 - invalid or excess argument(s)
# 3 - unimplemented feature (e.g. "reload")
# 4 - insufficient privilege
# 5 - program / file is not installed / present
# 6 - program is not configured
# 7 - program is not running
#
#
# Definitions
#
# Error Flag
BERROR=0

# for filecopy
LIBNAME=libctscm500-01.03.so
DEPNAME=libudev.so.0
LNKNAME=libctscm500.so
BIN_PATH=/usr/bin
ACC_PATH=/usr/share
LIB_PATH=/usr/lib
INC_PATH=/usr/include

# for UDEV
UDEV_SRC="40-cardterms.rules"
UDEV_OLD="95-cardterms.rules"

# for usb.ids
VENDORID=04E6
DEVICEID=1A02
VENDORNAME="SCM Microsystems"
DEVICENAME="eHealth500"

# show welcome
#
echo -e		"\n****************************************************************************"
echo -e		"****          Installscript der CT-API Bibliothek für das               ****"
echo -e		"****       SCM Microsystems GmbH eHealth500 mobiles Terminal            ****"
echo -e     "* Die Installation der benötigten Dateien und Erweiterung des udev-Service *"
echo -e     "****                     benötigt Root-Rechte                           ****"
echo -e		"****************************************************************************\n"

# Check for Root
#
echo -e     "****             Schritt 1: Überprüfung Root-Rechte                     ****\n"
if [ $EUID -ne 0 ]; then
	echo -e "\nFehler: Sie haben keine Root-Rechte!"
	exit 4
	else
	echo -e "\nRoot-Rechte sind zugewiesen."
fi

# add UDEV rule
echo -e     "\n****            Schritt 2: Überprüfung UDEV-Regeln                     ****\n"
UDEV_PATH=`find /etc -type d -name rules.d`
if [ -f $UDEV_PATH/$UDEV_SRC ]; then
    echo -e "\nUDEV-Regel bereits vorhanden."
    echo -n "Teste auf Unterschiede..."
    DIFFPATH=`which diff`
    DIFFS=`diff -q $UDEV_SRC $UDEV_PATH/$UDEV_SRC`
    if [ -n "$DIFFS" -o -z "$DIFFPATH" ]; then
	echo -n "Passe UDEV-Regel an..."
	cp -f ./$UDEV_SRC $UDEV_PATH
	if [ $? -ne 0 ]; then
	    echo "nicht erfolgreich!"
	    echo "Sind Schreibrechte in $UDEV_PATH zugewiesen?"
	    BERROR=1
	else
	    echo "OK."
	fi
    else 
	echo "Richtige Regel vorhanden!"
    fi
else
    echo -n "Installiere udev-Regel..."
    cp ./$UDEV_SRC $UDEV_PATH
    if [ $? -ne 0 ]; then
	echo "nicht erfolgreich!"
	echo "Sind Schreibrechte in $UDEV_PATH zugewiesen?"
	BERROR=1
    else
	echo "OK."
    fi
fi
if [ -f $UDEV_PATH/$UDEV_OLD ]; then
    echo -n "Weitere alte Regel gefunden, entferne diese..."
    rm -f $UDEV_PATH/$UDEV_OLD
    echo "OK."
fi

# add our entry to usb.ids
echo -e     "\n****     Schritt 3: Überprüfung usb.ids                 ****\n"
LOCATE=`which locate`
if [ -n "$LOCATE" ]; then
    USBIDSCOUNT=`locate -c -e usb.ids`
    if [ -n "$USBIDSCOUNT" ]; then
	USBIDS=`locate -n 1 -e usb.ids`
	if [ -h "$USBIDS" ]; then
	     USBIDS="$(ls -l $USBIDS | awk '{print $NF};')"
	fi
    fi
fi
if [ -z "$USBIDS" ]; then
USBIDS=`find /usr -type f -name usb.ids`
  if [ "$USBIDS" = "" ]; then
     USBIDS=`find /var -type f -name usb.ids`
     if [ "$USBIDS" = "" ]; then
        exit 5
     fi
  fi
fi

echo $USBIDS
test -e $USBIDS || exit 5
test -w $USBIDS || exit 4

TRUE=1
FALSE=0
INSERTDEV=$TRUE
INSERTVEN=$TRUE

echo "Prüfe auf Vendor ID in usb.ids:"
if [ $(grep -c "^$VENDORID" $USBIDS ) = 1 ]; then
    VIDLINE=`grep -n "^$VENDORID" $USBIDS | cut -d: -f1`
    echo "Vendor ID gefunden in Zeile: $VIDLINE."
    INSERTVEN=$FALSE
    if [ $(cat $USBIDS | sed -n "$VIDLINE,/^[0-9a-f]/p" | grep -c "$DEVICEID") = 1 ]; then
        echo "medline Device ID gefunden in $USBIDS."
	INSERTDEV=$FALSE
    else 
	if [ $(cat $USBIDS | sed -n "$VIDLINE,/^[0-9a-f]/p" | grep -c "$DEVICEID") -ge 2 ]; then  
	    echo "Warnung: Mehrfache $DEVICENAME gefunden in $USBIDS."
	    echo "Bitte prüfen Sie die Datei $USBIDS manuell!"
	    BERROR=1
            INSERTDEV=$FALSE
            INSERTVEN=$FALSE
        fi;
    fi;
else
    if [ $(grep -c "^$VENDORID" $USBIDS ) -ge 2 ]; then 
	echo "Warnung: Mehrfache $VENDORNAME gefunden in $USBIDS."
	echo "Bitte prüfen Sie die Datei $USIDS manuell!"
	BERROR=1
        INSERTDEV=$FALSE
        INSERTVEN=$FALSE
    fi;
fi;

if [ $INSERTVEN -eq $TRUE ]; then
    echo "Ergänze VendorID."
    VIDS="$(cat $USBIDS | grep "^[0-9a-f]" | awk '{print $1}')"
    let DEC_VENDOR="0x$VENDORID"
    rm -f $USSBIDS.tmp
    for vid in $VIDS 
    do
        let dec_vid="0x$vid"
        if [ $dec_vid -ge $DEC_VENDOR ]; then
	   sed "/^$vid/i$VENDORID  $VENDORNAME" $USBIDS > $USBIDS.tmp
	   break
	fi;
    done
    rm -f $USBIDS
    mv $USBIDS.tmp $USBIDS 
fi;

if [ $INSERTDEV -eq $TRUE ]; then
    echo "Ergänze DeviceID."
    rm -f $USBIDS.tmp
    sed "/^$VENDORID/a\
\	$DEVICEID  $DEVICENAME" $USBIDS > $USBIDS.tmp
    rm -f $USBIDS
    mv $USBIDS.tmp $USBIDS 
fi;

# Copy and link library
echo -e     "\n****     Schritt 4: Kopiere Library und erzeuge Links   ****\n"
if [ -f $LIB_PATH/$LIBNAME ]; then
    echo "Library bereits vorhanden."
else
    echo -n "Kopiere Library und lege Link an..."
    cp -f $LIBNAME $LIB_PATH
    if [ $? -ne 0 ]; then
	echo "nicht erfolgreich!"
	echo "Sind Schreibrechte in $LIB_PATH zugewiesen?"
	BERROR=1
    else
	echo "OK."
	OLD_PWD=$PWD
	cd $LIB_PATH
	ln -sf $LIBNAME $LNKNAME
	cd $OLD_PWD
    fi
fi

echo -e     "\n****     Schritt 5: Überprüfe Abhängigkeiten            ****\n"
DEPS=`ldd $LIBNAME | grep "not found" | awk '{print $1;}'`
if [ -n "$DEPS" ]; then
    echo "Eine benötigte Abhängigkeit ist nicht vorhanden ($DEPS)."
    if [ "$DEPS" == "$DEPNAME" ]; then
	if [ -f /etc/lsb-release ]; then
		source /etc/lsb-release
	else
		if [ -f /etc/debian_version ]; then
			DISTRIB_ID="Debian"
			DISTRIB_RELEASE=`cat /etc/debian_version`
		fi
	fi
	
	echo "Folgende Distribution wurde gefunden: $DISTRIB_ID Version: $DISTRIB_RELEASE"
	if [ "$DISTRIB_ID" == "Ubuntu" -a "$DISTRIB_RELEASE" == "8.04" ]; then
	    echo -e "\n\n******************************************************************"
	    echo -e     "* Ubuntu 8.04 enthält die benötigte Library libudev nicht.       *"
	    echo -e     "* Es wird dringend empfohlen die mitgelieferte Libary zu nutzen. *"
	    echo -e     "******************************************************************\n"
	elif [ "$DISTRIB_ID" == "Debian" -a "$DISTRIB_RELEASE" == "5.0.3" ]; then
	    echo -e "\n\n********************************************************************"
	    echo -e     "* Debian \"Lenny\" enthält die benötigte Library libudev noch nicht. *"
	    echo -e     "* Es wird empfohlen die in \"Squeeze\" enthaltene Version zu nutzen. *"
	    echo -e     "********************************************************************\n"
	fi
	echo -n "Wollen Sie die mitgelieferte Library installieren (j/n): "
	read -n 1 SELECT
	if [ "$SELECT" == "j" ]; then
	    echo -n -e "\nInstalliere die benötigte Datei in $LIB_PATH..."
	    cp $DEPNAME $LIB_PATH
	    if [ $? -ne 0 ]; then
		echo "nicht erfolgreich!"
		echo "Sind Schreibrechte in $LIB_PATH zugewiesen?"
		BERROR=1
	    else
		echo "OK."
	    fi
	else 
	    echo -e "\nInstalliere die benötigte Datei NICHT."	    
	    echo  "Bitte installieren Sie sich die benötigte Datei selbstständig"	    
	    echo  "mit Hilfe Ihres Paketmanagers."	    
	    BERROR=5
	fi
    fi
fi

ldconfig


echo -n -e "\nExit: $BERROR"
if [ $BERROR -eq 0 ]; then
    echo -e "\nInstallation erfolgreich!"
else
    echo -e -n "\n\aFehler bei der Installation, bitte prüfen Sie"
    echo " die Bildschirmausgaben!"
fi


echo -e "\n"
exit 0
